// Canvas
// draws lines, circles, ellipses, and rectangles
// 2000.02.10

// Copyright (C) 2000 Guoyichao (fsl@citiz.net)
// Modified by Dan Steinman; converted to Canvas improved fills(), added drawRect(), fillRect()
// Modified by Rob Breeds: improve performance on ellipse and circle algorithms, and added support for different line thicknesses
// Distributed under the terms of the GNU Library General Public License
// Available at http://www.dansteinman.com/dynapi/

Canvas = function() {
	this.component = new DynLayer()
}
WidgetInterface.inherit(Canvas)
Canvas.prototype.color = "black"
Canvas.prototype.thickness = 1
Canvas.prototype.setPaintComponent = function(c) {
	this.component = c
}
Canvas.prototype.setPaintColor = function(color) {
	this.color = color
}
Canvas.prototype.setPaintThickness = function(t) {
	this.thickness = t
}
Canvas.prototype.drawPixel = function(x,y) {
	drawPixel(x,y,this.color,this.thickness,this.component)
}
Canvas.prototype.drawLine = function(x1,y1,x2,y2) {
	drawLine(x1,y1,x2,y2,this.color,this.thickness,this.component)
}
Canvas.prototype.drawCircle = function(x,y,r) {
	drawCircle(x+r,y+r,r,this.color,this.thickness,this.component)
}
Canvas.prototype.fillCircle = function(x,y,r) {
	fillCircle(x+r,y+r,r,this.color,this.component)
}
Canvas.prototype.drawEllipse = function(x,y,w,h) {
	drawEllipse(x+w/2,y+h/2,w,h,this.color,false,this.thickness,this.component)
}
Canvas.prototype.fillEllipse = function(x,y,w,h) {
	drawEllipse(x+w/2,y+h/2,w,h,this.color,true,this.thickness,this.component)
}
Canvas.prototype.drawRect = function(x,y,w,h) {
	drawRect(x,y,w,h,this.color,this.thickness,this.component)
}
Canvas.prototype.fillRect = function(x,y,w,h) {
	fillRect(x,y,w,h,this.color,this.component)
}
Canvas.prototype.clear = function() {
	if (this.component.getClass()==DynLayer) {
		this.component.deleteAllChildren()
		this.component.setHTML()
	}
}

// Drawing Routines

function drawPixel(x,y,color,t,parent) {
	parent.addChild( new DynLayer('',Math.round(x),Math.round(y),t,t,color) )
}

function drawLine(x1,y1,x2,y2,color,t,parent) {
	var flag = (Math.abs(y2-y1) > Math.abs(x2-x1))
	var dx,dy,x,y,e,xstep,ystep
	if (flag) dx=Math.abs(y2-y1),dy=Math.abs(x2-x1),x=y1,y=x1
	else dx=Math.abs(x2-x1),dy=Math.abs(y2-y1),x=x1,y=y1
	xstep=x1>x2?-1:1
	ystep=y1>y2?-1:1
	if(x1==x2||y1==y2) {
		if(x1==x2) {
			var ny1 = Math.min(y1,y2)
			var ny2 = Math.max(y1,y2)
			parent.addChild( new DynLayer('',x1,ny1,t,(ny2-ny1)*t,color) )
			return
		}
		else {
			var nx1 = Math.min(x1,x2)
			var nx2 = Math.max(x1,x2)
			parent.addChild( new DynLayer('',nx1,y1,(nx2-nx1)*t,t,color) )
			return
		}
	}
	// Bresenham Method Begin
	var e=-dx
	for(var count=0;count<=dx;count++) {
		if (flag) drawPixel(y,x,color,t,parent)
		else drawPixel(x,y,color,t,parent)
		if (flag) x+=ystep
		else x+=xstep
		e+=dy<<1
		if(e>=0) {
			if(flag) y+=xstep
			else y+=ystep
			e-=dx<<1
		}
	}
	return
}

function drawCircle(x,y,radius,color,t,parent) {
	var cx = 0
	var cy = radius
	var df = 1 - radius
	var d_e = 3
	var d_se = -(radius<<1) + 5
	do {
		drawPixel(x+cx, y+cy, color,t,parent)
		if (cx) drawPixel(x-cx, y+cy, color,t,parent)
      if (cy) 	drawPixel(x+cx, y-cy, color,t,parent)
      if ((cx) && (cy)) drawPixel(x-cx, y-cy, color,t,parent) 
      if (cx != cy) {
			drawPixel(x+cy, y+cx, color,t,parent)
			if (cx) drawPixel(x+cy, y-cx, color,t,parent)
			if (cy) drawPixel(x-cy, y+cx, color,t,parent)
			if (cx && cy) drawPixel(x-cy, y-cx, color,t,parent)
      }
      if (df < 0)  {
			df += d_e
			d_e += 2
			d_se += 2
      }
      else { 
			df += d_se
			d_e += 2
			d_se += 4
			cy--
		}
		cx++
   } while (cx <= cy)
}

function fillCircle(x,y,radius,color,parent) {
	var cx = 0
	var cy = radius
	var df = 1 - radius
	var d_e = 3
	var d_se = -(radius<<1) + 5
	do {
		fillRect(x-cy, y-cx, cy<<1, cx<<1, color,parent)
      if (df < 0)  {
			df += d_e
			d_e += 2
			d_se += 2
		}
      else {
			if (cx != cy) fillRect(x-cx, y-cy, cx<<1, cy<<1, color,parent)
			df += d_se
			d_e += 2
			d_se += 4
			cy--
		}
		cx++
   } while (cx <= cy)
}

function drawEllipse(cx,cy,rx,ry,color,filled,t,parent) {
	var x,y,a2,b2, S, T
	a2 = rx*rx
	b2 = ry*ry
	x = 0
	y = ry
	S = a2*(1-2*ry) + 2*b2
	T = b2 - 2*a2*(2*ry-1)
	symmPaint(cx,cy,x,y,color,filled,t,parent)
	do {
		if (S<0) {
			S += 2*b2*(2*x+3)
			T += 4*b2*(x+1)
			x++
		} else if (T<0) {
			S += 2*b2*(2*x+3) - 4*a2*(y-1)
			T += 4*b2*(x+1) - 2*a2*(2*y-3)
			x++
			y--
		} else {
			S -= 4*a2*(y-1)
			T -= 2*a2*(2*y-3)
 			y--
		}
		symmPaint(cx,cy,x,y,color,filled,t,parent)
	} while (y>0)
}

//Bresenham's algorithm for ellipses
function symmPaint(cx,cy, x, y, color, filled, t, parent){
	if (filled) { 
		fillRect ( cx-x, cy-y, x<<1, y<<1, color, parent )
	} else {
		drawPixel ( cx-x, cy-y, color, t,parent )
		drawPixel ( cx+x, cy-y, color, t,parent )
		drawPixel ( cx-x, cy+y, color, t,parent )
		drawPixel ( cx+x, cy+y, color, t,parent )
	}
}

function drawRect(x,y,w,h,color,t,parent) {
	parent.addChild( new DynLayer('',x,y,w,t,color) )
	parent.addChild( new DynLayer('',x,y,t,h,color) )
	parent.addChild( new DynLayer('',x+w-t,y,t,h,color) )
	parent.addChild( new DynLayer('',x,y+h-t,w,t,color) )
}

function fillRect(x,y,w,h,color,parent) {
	parent.addChild( new DynLayer('',x,y,w,h,color) )
}


// some old code used as wrappers
/*

function Pixel(x,y,color,t) {
	this.x = x
	this.y = y
	this.color = color
	this.t = t
}
Pixel.prototype.create = function(parent) {
	drawPixel(this.x,this.y,this.color,this.t)
}

function Line(x1,y1,x2,y2,color,t) {
	this.x1 = x1
	this.y1 = y1
	this.x2 = x2
	this.y2 = y2
	this.color = color
	this.t = t
}
Line.prototype.create = function(parent) {
	var nx1 = Math.min(this.x1,this.x2)
	var nx2 = Math.max(this.x1,this.x2)
	var ny1 = Math.min(this.y1,this.y2)
	var ny2 = Math.max(this.y1,this.y2)
	if(this.x1==this.x2||this.y1==this.y2) {
		if(x1==x2) this.dlyr = createDynLayer('',null,parent,nx1,ny1,t,ny2-ny1,color)
		else this.dlyr = createDynLayer('',null,parent,nx1,ny1,nx2-nx1,t,color)
	}
	else {
		var w = nx2-nx1
		var h = ny2-ny1
		this.dlyr = createDynLayer('',null,parent,nx1,ny1,w+this.t,h+this.t)
		drawLine(this.x1-nx1,this.y1-ny1,this.x2-nx1,this.y2-ny1,this.color,this.t,this.dlyr)
	}
}

function Rect(x,y,width,height,color,filled,t) {
	this.x = x
	this.y = y
	this.w = width
	this.h = height
	this.color = color
	this.filled = filled
	this.t = t
}
Rect.prototype.create = function(parent) {
	this.dlyr = createDynLayer('',null,parent,this.x,this.y,this.w,this.h)
	if (this.filled) fillRect(0,0,this.w,this.h,this.color,this.dlyr)
	else drawRect(0,0,this.w,this.h,this.color,this.t,this.dlyr)
}

function Ellipse(x,y,width,height,color,filled,t) {
	this.x = x
	this.y = y
	this.w = width
	this.h = height
	this.color = color
	this.filled = filled
	this.t = t
}
Ellipse.prototype.create = function(parent) {
	if (this.filled) this.dlyr = createDynLayer('',null,parent,this.x,this.y,this.w,this.h)
	else this.dlyr = createDynLayer('',null,parent,this.x,this.y,this.w+this.t,this.h+this.t)

	if (this.w == this.h) {
		var r = this.w/2-1
		if (this.filled) fillCircle(r,r,r,this.color,this.dlyr)
		else drawCircle(r,r,r,this.color,this.t,this.dlyr)
	}
	else {
		var h = this.w/2-1
		var v = this.h/2-1
		if (this.filled) drawEllipse(h,v,h,v,this.color,true,null,this.dlyr)
		else drawEllipse(h,v,h,v,this.color,false,this.t,this.dlyr)
	}
}

*/
