// List and ListItem
// a generic text selection widget
// 2000.06.01

// Copyright (C) 2000 Dan Steinman
// Redistributable under the terms of the GNU Library General Public License
// Available at http://www.dansteinman.com/dynapi/

function List() {
	this.items = []
	this.component = new DynLayer()
	this.component.target = this
	this.component.onCreate = function() {
		var o = this.target
		o.maxWidth = 0
		o.totalHeight = 0
		var dw = o.component.w
		if (!dw) {
			for (i in o.items) {
				var item = o.items[i]
				var w = item.getComponent().w
				var h = item.getComponent().h
				if (w>o.maxWidth) o.maxWidth=w
			}
			dw = o.maxWidth
		}
		o.setWidth(dw)
	}
	this.itemListener = new EventListener(this)
	this.itemListener.onselect = function(e) {
		e.getTarget().itemOnSelect(e.getSource())
	}
	this.itemListener.ondeselect = function(e) {
		e.getTarget().itemOnDeselect(e.getSource())
	}
}

WidgetInterface.inherit(List)

List.prototype.style = new Object()
List.prototype.style.bgColor = '#ffffff'
List.prototype.style.bgColorRoll = '#eaebff'
List.prototype.style.bgColorSelect = '#cdceff'
List.prototype.style.textColor = '#000000'
List.prototype.style.textColorRoll = '#000000'
List.prototype.style.textColorSelect = '#000000'
List.prototype.style.padding = 3

List.prototype.nullItem = {}
List.prototype.nullItem.text = ""
List.prototype.nullItem.value = ""
List.prototype.multiMode = false
List.prototype.selectedIndex = -1
List.prototype.selectedItem = List.prototype.nullItem
List.prototype.add = function(item) {
	if (item.getClass()==ListItem) {
		var i = this.items.length
		this.items[i] = item
		item.isItem = true
		item.target = this

		item.addEventListener(this.itemListener)

		this.component.addChild(item)
		if (this.component.created) {
			if (i>0) item.getComponent().setY(this.items[i-1].getComponent().y+this.items[i-1].getComponent().h)
			this.totalHeight += item.getComponent().h
			
			if (!item.wrap && item.getComponent().w>this.maxWidth) {
				this.maxWidth = item.getComponent().w
				this.setWidth(this.maxWidth)
				this.component.setSize(null,this.totalHeight)
			}
			else {
				item.setWidth(this.component.w)
				this.component.setSize(null,this.totalHeight)
			}
		}
	}
	return item
}
List.prototype.remove = function(item) {
	var i = this.indexOf(item)
	if (i==-1) return
	this.items[i].getComponent().deleteFromParent()
	for (var j=i;j<this.items.length-1;j++) {
		this.items[j] = this.items[j+1]
	}
	this.items.length--
	if (i==0) this.items[0].component.setY(0)
	this.setWidth(this.component.w)
	if (!this.multiMode) {
		this.selectedIndex = -1
		this.selectedItem = this.nullItem
	}
}
List.prototype.clear = function() {
	for (var i in this.items) {
		this.items[i].getComponent().deleteFromParent()
	}
	this.items = new Array()
	this.totalHeight = 0
	this.component.setSize(null,0)
	this.selectedIndex = -1
	this.selectedItem = this.nullItem
}
List.prototype.indexOf = function(item) {
	for (var i in this.items) {
		if (this.items[i] == item) return i
	}
	return -1
}
List.prototype.setWidth = function(w) {
	if (w==null) {
		if (this.maxWidth>0) w = this.maxWidth
		else return
	}
	this.totalHeight = 0
	for (var i in this.items) {
		this.items[i].setWidth(w)
		if (i!=0) this.items[i].getComponent().setY(this.items[i-1].getComponent().y+this.items[i-1].getComponent().h)
		this.totalHeight += this.items[i].getComponent().h
	}
	this.component.setSize(w,this.totalHeight)
}
List.prototype.itemOnSelect = function(item) {	
	this.selectedIndex = this.indexOf(item)
	if (this.selectedIndex == -1) {
		this.selectedItem = this.nullItem
	}
	else {
		this.selectedItem = item
		for (var i in this.items) {
			if (this.items[i]!=item && i!=this.selectedIndex && this.items[i].selected && !this.multiMode) this.items[i].setSelected(false)
		}
		this.invokeEvent("select")
	}
}
List.prototype.itemOnDeselect = function(item) {
	if (this.selectedItem == item) {
		this.selectedIndex = -1
		this.selectedItem = this.nullItem
	}
}
List.prototype.setSelectionMode = function(mode) {
	for (i in this.items) {
		if (this.items[i].selected) this.items[i].setSelected(false)
	}
	this.multiMode = mode
}
List.prototype.getSelectedIndex = function() {
	return this.selectedIndex
}
List.prototype.getSelectedItem= function() {
	return this.selectedItem
}
List.prototype.getSelectedIndexes = function() {
	var a = new Array()
	for (i in this.items) {
		if (this.items[i].selected) a[a.length] = i
	}
	return a
}


function ListItem(text,value,wrap) {
	this.value = value
	this.text = text
	
	this.label = new Label(text,wrap)
	this.label.addCover()
	this.label.setPadding(this.style.padding)

	this.component = this.label.component
	this.component.setParentComponent(this)
	this.component.captureMouseEvents()
	this.component.setBgColor(this.style.bgColor)

	this.events = new EventListener(this)
	this.events.onmousedown = function (e) {
		var o = e.getTarget()
		o.setSelected(!o.selected)
	}
	this.events.onmouseover = function (e) {
		var o = e.getTarget()
		if (!o.selected) o.label.component.setBgColor(o.style.bgColorRoll)
		o.isMouseOver = true
	}
	this.events.onmouseout = function (e) {
		var o = e.getTarget()
		if (!o.selected) o.label.component.setBgColor(o.style.bgColor)
		o.isMouseOver = false
	}	
	this.addEventListener(this.events)
}

WidgetInterface.inherit(ListItem)

ListItem.prototype.deselectable = true
ListItem.prototype.isMouseOver = false
ListItem.prototype.selected = false
ListItem.prototype.style = List.prototype.style

ListItem.prototype.setItemStyle = function(style) {
	this.style = style
	this.label.component.setBgColor(this.style.bgColor)
}
ListItem.prototype.setSelected = function(b) {
    this.selected=b;
	if (this.selected) {
		this.label.component.setBgColor(this.style.bgColorSelect)
		this.invokeEvent("select")
	}
	else if (this.deselectable) {
		this.label.component.setBgColor(this.isMouseOver?this.style.bgColorRoll:this.style.bgColor)
		this.invokeEvent("deselect")
	}
}
ListItem.prototype.setValue = function(value) {this.value = value}
ListItem.prototype.getValue = function() {return this.value}

ListItem.prototype.setWidth = function(w) {this.label.setWidth(w)}
ListItem.prototype.setHeight = function(h) {this.label.setHeight(h)}
ListItem.prototype.setSize = function(w,h) {this.label.setSize(w,h)}
ListItem.prototype.setAlignment = function(a) {this.label.setAlignment(a)}
ListItem.prototype.setText = function(text) {this.label.setText(text)}
ListItem.prototype.getText = function(text) {return this.label.text}
ListItem.prototype.setValue = function(v) {this.value = v}
ListItem.prototype.getValue = function() {return this.value}
